@@ -2,7 +2,9 @@
 
 package Ingredient2Recipe;
 use strict;
+use warnings;
 use base qw( Data::ObjectDriver::BaseObject );
+use DodTestUtil;
 
 use Data::ObjectDriver::Driver::DBI;
 
@@ -11,7 +13,7 @@ __PACKAGE__->install_properties({
     datasource => 'ingredient2recipe',
     primary_key => [ 'recipe_id', 'ingredient_id', ],
     driver      => Data::ObjectDriver::Driver::DBI->new(
-            dsn      => 'dbi:SQLite:dbname=global.db',
+            dsn      => DodTestUtil::dsn('global'),
     ),
 });
 
